@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
-* Copyright (C) 2007-2011, International Business Machines Corporation and
-* others. All Rights Reserved.                                                *
+* Copyright (C) 2007-2016, International Business Machines Corporation and
+* others. All Rights Reserved.
 *******************************************************************************
 *
 * File DTPTNGEN.H
@@ -9,11 +9,12 @@
 *******************************************************************************
 */
 
-#include "uvector.h"
-
 #ifndef __DTPTNGEN_IMPL_H__
 #define __DTPTNGEN_IMPL_H__
 
+#include "unicode/udatpg.h"
+#include "uvector.h"
+
 // TODO(claireho): Split off Builder class.
 // TODO(claireho): If splitting off Builder class: As subclass or independent?
 
@@ -38,12 +39,14 @@
 #define DOT               ((UChar)0x002E)
 #define COLON             ((UChar)0x003A)
 #define CAP_A             ((UChar)0x0041)
+#define CAP_B             ((UChar)0x0042)
 #define CAP_C             ((UChar)0x0043)
 #define CAP_D             ((UChar)0x0044)
 #define CAP_E             ((UChar)0x0045)
 #define CAP_F             ((UChar)0x0046)
 #define CAP_G             ((UChar)0x0047)
 #define CAP_H             ((UChar)0x0048)
+#define CAP_J             ((UChar)0x004A)
 #define CAP_K             ((UChar)0x004B)
 #define CAP_L             ((UChar)0x004C)
 #define CAP_M             ((UChar)0x004D)
@@ -51,12 +54,15 @@
 #define CAP_Q             ((UChar)0x0051)
 #define CAP_S             ((UChar)0x0053)
 #define CAP_T             ((UChar)0x0054)
+#define CAP_U             ((UChar)0x0055)
 #define CAP_V             ((UChar)0x0056)
 #define CAP_W             ((UChar)0x0057)
+#define CAP_X             ((UChar)0x0058)
 #define CAP_Y             ((UChar)0x0059)
 #define CAP_Z             ((UChar)0x005A)
 #define LOWLINE           ((UChar)0x005F)
 #define LOW_A             ((UChar)0x0061)
+#define LOW_B             ((UChar)0x0062)
 #define LOW_C             ((UChar)0x0063)
 #define LOW_D             ((UChar)0x0064)
 #define LOW_E             ((UChar)0x0065)
@@ -78,6 +84,7 @@
 #define LOW_U             ((UChar)0x0075)
 #define LOW_V             ((UChar)0x0076)
 #define LOW_W             ((UChar)0x0077)
+#define LOW_X             ((UChar)0x0078)
 #define LOW_Y             ((UChar)0x0079)
 #define LOW_Z             ((UChar)0x007A)
 #define DT_SHORT          -0x102
@@ -141,12 +148,11 @@ public:
     FormatParser();
     virtual ~FormatParser();
     void set(const UnicodeString& patternString);
-    UBool isQuoteLiteral(const UnicodeString& s) const;
     void getQuoteLiteral(UnicodeString& quote, int32_t *itemIndex);
-    int32_t getCanonicalIndex(const UnicodeString& s) { return getCanonicalIndex(s, TRUE); }
-    int32_t getCanonicalIndex(const UnicodeString& s, UBool strict);
     UBool isPatternSeparator(UnicodeString& field);
-    void setFilter(UErrorCode &status);
+    static UBool isQuoteLiteral(const UnicodeString& s);
+    static int32_t getCanonicalIndex(const UnicodeString& s) { return getCanonicalIndex(s, TRUE); }
+    static int32_t getCanonicalIndex(const UnicodeString& s, UBool strict);
 
 private:
    typedef enum TokenStatus {
@@ -166,7 +172,7 @@ public:
     int32_t extraFieldMask;
 
     DistanceInfo() {}
-    virtual ~DistanceInfo() {}
+    virtual ~DistanceInfo();
     void clear() { missingFieldMask = extraFieldMask = 0; }
     void setTo(DistanceInfo& other);
     void addMissing(int32_t field) { missingFieldMask |= (1<<field); }
@@ -188,7 +194,7 @@ public:
     int32_t getDistance(const DateTimeMatcher& other, int32_t includeMask, DistanceInfo& distanceInfo);
     DateTimeMatcher();
     DateTimeMatcher(const DateTimeMatcher& other);
-    virtual ~DateTimeMatcher() {}
+    virtual ~DateTimeMatcher();
     int32_t getFieldMask();
 };
 
