@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2010, International Business Machines Corporation and
+ * Copyright (c) 1997-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -30,6 +30,7 @@
 #include "ccolltst.h"
 #include "callcoll.h"
 #include "cfrtst.h"
+#include "cmemory.h"
 #include "unicode/ustring.h"
 #include "string.h"
 
@@ -182,7 +183,7 @@ static void TestSecondary()
     }
     log_verbose("Testing fr_CA Collation with Secondary strength\n");
     /*test acute and grave ordering (compare to french collation)*/
-    testAcuteSize = sizeof(testAcute) / sizeof(testAcute[0]);
+    testAcuteSize = UPRV_LENGTHOF(testAcute);
     for (i = 0; i < testAcuteSize; i++)
     {
         for (j = 0; j < testAcuteSize; j++)
@@ -320,7 +321,7 @@ static void TestGetSortKey() {
 
     lenActualSortKey = ucol_getSortKey(pCollator,
         (const UChar *)pucUTF16,
-        sizeof(pucUTF16) / sizeof(pucUTF16[0]),
+        UPRV_LENGTHOF(pucUTF16),
         pucSortKey,
         LENSORTKEY);
 
