@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 2002-2014, International Business Machines Corporation and
+ * Copyright (c) 2002-2016, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************
  *
@@ -13,6 +13,7 @@
 #if !UCONFIG_NO_NORMALIZATION
 
 #include "intltest.h"
+#include "cmemory.h"
 #include "cstring.h"
 #include "canittst.h"
 #include "unicode/caniter.h"
@@ -20,8 +21,6 @@
 #include "unicode/uchar.h"
 #include "hash.h"
 
-#define ARRAY_LENGTH(array) ((int32_t)(sizeof (array) / sizeof (*array)))
-
 #define CASE(id,test) case id:                          \
                           name = #test;                 \
                           if (exec) {                   \
@@ -148,7 +147,7 @@ void CanonicalIteratorTest::TestBasic() {
     int32_t i = 0;
     CanonicalIterator it("", status);
     if(U_SUCCESS(status)) {
-      for (i = 0; i < ARRAY_LENGTH(testArray); ++i) {
+      for (i = 0; i < UPRV_LENGTHOF(testArray); ++i) {
           //logln("Results for: " + name.transliterate(testArray[i]));
           UnicodeString testStr = CharsToUnicodeString(testArray[i][0]);
           it.setSource(testStr, status);
